LFRtools = {};
LFRtools.frame = CreateFrame("Frame");
LFRtools.frame:RegisterEvent("ADDON_LOADED");
LFRtools.frame:RegisterEvent("GROUP_JOINED");
--LFRtools.frame:RegisterEvent("GROUP_ROSTER_UPDATE");
LFRtools.frame:RegisterEvent("INSPECT_READY");
LFRtools.frame:RegisterEvent("PARTY_LEADER_CHANGED");
LFRtools.frame:RegisterEvent("PARTY_MEMBER_ENABLE");
LFRtools.frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
LFRtools.frame:RegisterEvent("ROLE_CHANGED_INFORM");
LFRtools.frame:RegisterEvent("UNIT_CONNECTION");
LFRtools.frame:RegisterEvent("PLAYER_ALIVE");
LFRtools.frame:RegisterEvent("READY_CHECK");
LFRtools.queue = {};
LFRtools.check = {};

function LFRtools.onEvent(frame, event, arg, ...)
	if (event == "INSPECT_READY") then
		local _, _, _, _, _, name, realm = GetPlayerInfoByGUID(arg);
		if (realm) and (realm ~= "") then
			name = name .. "-" .. realm;
		end
		for i, entry in ipairs(LFRtools.queue) do
			if (name == entry) then
				table.remove(LFRtools.queue, i);
				LFRtools.inspect(entry);
				ClearInspectPlayer();
				LFRtools.processQueue();
				return;
			end
		end
	--elseif (event == "GROUP_ROSTER_UPDATE") then
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: GROUP_ROSTER_UPDATE" .. (arg or ""));
	elseif (event == "ROLE_CHANGED_INFORM") then
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: ROLE_CHANGED_INFORM " .. arg);
		local officer = ...;
		if (not UnitIsUnit(officer, "player")) then
			LFRtools.enqueue(arg);
		end
	elseif (event == "UNIT_CONNECTION") then
		local connected = ...;
		if (connected) then
			--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: UNIT_CONNECTION " .. arg);
			LFRtools.enqueue(arg);
		end
	elseif (event == "PARTY_MEMBER_ENABLE") then
--		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: PARTY_MEMBER_ENABLE " .. arg);
		LFRtools.enqueue(arg);
	elseif (event == "PLAYER_SPECIALIZATION_CHANGED") then
		--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: PLAYER_SPECIALIZATION_CHANGED " .. arg);
		if (arg ~= nil) then
			LFRtools.enqueue(arg);
		end
	elseif (event == "GROUP_JOINED") then
		LFRtools.check = {};
		LFRtools.scanGroup();
	elseif (event == "PARTY_LEADER_CHANGED") then
		if (UnitIsGroupLeader("player")) then
			LFRtools.scanGroup();
		end
	elseif (event == "READY_CHECK") then
		local chatType = IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and "INSTANCE_CHAT" or (IsInRaid() and "RAID" or "PARTY");
		local msg = "", raidRole;
		for i, entry in ipairs(LFRtools.check) do
			if (UnitInParty(entry) or UnitInRaid(entry)) then
				raidRole = UnitGroupRolesAssigned(entry);
				if (entry ~= "NONE") then
					msg = msg .. LFRtools.name(entry) .. " (" .. raidRole .. "), "
				end
			end
		end
		if (msg ~= "") then
			SendChatMessage("LFRtools: " .. msg .. "is/are not in their correct specialization.", chatType);
		end
	elseif (event == "PLAYER_ALIVE") then
		if (#LFRtools.queue > 0) then
			DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue resumed");
			LFRtools.processQueue();
		end
	elseif (event == "ADDON_LOADED") and (arg == "LFRtools") then
		LFRtools.frame:UnregisterEvent("ADDON_LOADED");
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Addon loaded");
		if (IsInGroup()) then
			LFRtools.scanGroup();
		end
	end
end
LFRtools.frame:SetScript("OnEvent", LFRtools.onEvent);

function LFRtools.inspect(unit)
	--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Inspecting " .. LFRtools.name(unit));
	local raidRole = UnitGroupRolesAssigned(unit);
	local _, _, _, _, _, specRole = GetSpecializationInfoByID(GetInspectSpecialization(unit));
	local chatType = IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and "INSTANCE_CHAT" or (IsInRaid() and "RAID" or "PARTY");
	if (IsInGroup(LE_PARTY_CATEGORY_HOME)) and ((UnitIsGroupLeader("player")) or (UnitIsRaidOfficer("player"))) then
		UnitSetRole(unit, specRole);
		return;
	end
	if (raidRole == "NONE") then
		return;
	elseif (raidRole ~= specRole) then
		SendChatMessage("LFRtools: " .. LFRtools.name(unit) .. " signed up as a " .. raidRole .. " but is actually a " .. specRole .. ".", chatType);
		for i, entry in ipairs(LFRtools.check) do
			if (unit == entry) then
				return;
			end
		end
		table.insert(LFRtools.check, unit);
	else
		for i, entry in ipairs(LFRtools.check) do
			if (unit == entry) then
				local sex = UnitSex(unit);
				if (sex == 2) then
					sex = "his";
				elseif (sex == 3) then
					sex = "her";
				else
					sex = "its";
				end
				SendChatMessage("LFRtools: " .. LFRtools.name(unit) .. " has corrected " .. sex .. " specialization to " .. specRole .. ".", chatType);
				table.remove(LFRtools.check, i);
				return;
			end
		end
	end
end

function LFRtools.scanGroup()
	--local spec;
	local unitType = IsInRaid() and "raid" or "party";
	for i = 1, GetNumGroupMembers() do
		--spec = GetInspectSpecialization(unitType .. i);
		--if (spec == 0) then
			LFRtools.enqueue(unitType .. i);
		--else
			--LFRtools.inspect(unitType .. i);
		--end
	end
	if (#LFRtools.queue > 0) then
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queued " .. #LFRtools.queue);
	end
end

function LFRtools.enqueue(unit)
	local canBeTank, canBeHealer, canBeDPS = UnitGetAvailableRoles(unit);
	if (not canBeTank) and (not canBeHealer) and (canBeDPS) then
		if (IsInGroup(LE_PARTY_CATEGORY_HOME)) and ((UnitIsGroupLeader("player")) or (UnitIsRaidOfficer("player"))) then
			UnitSetRole(unit, "DAMAGER");
		end
		return;
	end
	local name, realm = UnitName(unit);
	if (name == nil) or (name == "Unknown") then
		return;
	elseif (realm) and (realm ~= "") then
		name = name .. "-" .. realm;
	end
	for i, entry in ipairs(LFRtools.queue) do
		if (name == entry) then
			return;
		end
	end
	--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queued " .. LFRtools.name(unit));
	table.insert(LFRtools.queue, name);
	if (not UnitIsDead("player")) and (not LFRtools.frame:GetScript("OnUpdate")) then
		LFRtools.processQueue();
	end
end

function LFRtools.name(unit)
	local name, realm = UnitName(unit);
	if (not realm) or (realm == "") then
		realm = GetRealmName();
	end
	return name .. "-" .. realm;
end

function LFRtools.onUpdate(frame, elapsed)
	LFRtools.total = LFRtools.total + elapsed;
	if (LFRtools.total >= 5) then --Timeout
		table.insert(LFRtools.queue, table.remove(LFRtools.queue, 1)); --Shuffle
		LFRtools.processQueue();
	end
end

function LFRtools.processQueue()
	if (table.getn(LFRtools.queue) ~= 0) then
		if (UnitIsDead("player")) then
			LFRtools.frame:SetScript("OnUpdate", nil);
			DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue paused");
			return;
		end
		for i, entry in ipairs(LFRtools.queue) do
			if (UnitIsConnected(entry)) and ((UnitInParty(entry)) or (UnitInRaid(entry)) --[[or (UnitIsUnit(entry, "player"))--]]) then
				if (CanInspect(entry)) then
					LFRtools.total = 0; --Reset timeout
					LFRtools.frame:SetScript("OnUpdate", LFRtools.onUpdate);
					NotifyInspect(entry);
					--DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Requesting " .. LFRtools.name(entry));
					return;
				end
			else
				table.remove(LFRtools.queue, i);
				LFRtools.processQueue();
				return;
			end
		end
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: None of queued in range");
	else
		LFRtools.frame:SetScript("OnUpdate", nil);
		DEFAULT_CHAT_FRAME:AddMessage("LFRtools: Queue done");
	end
end
